/** @file fifo_scen.cpp
 * This file contains the scenario class implementation.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <fifo_scen.h>

#include <tracer/tracer.hpp>

namespace cpptesk {
namespace examples {
namespace fifo {

bool FIFOScenario::init(int argc, char ** argv)
{
    UNUSED_ARG(argc);
    UNUSED_ARG(argv);

    // Initialize reference model
    dut.initialize();
    return true;
}

void FIFOScenario::finish()
{
    // Finalize reference model
    dut.finalize();
}

bool FIFOScenario::scen_nop(Ctx& ctx)
{
    // Example of iteration variables declaration:
    // int& param1 = IVAR(a);
    // int& param2 = IVAR(b);

    // All the local variables should be declared before IBEGIN

    // Each scenario function begins from IBEGIN
    IBEGIN
    // Iteration cycles might be situated right after IBEGIN
    // for(param1 = 0; param1 < 2; param1++) {
    //    for(param2 = 0; param2 < 2; param2++) {

    // After the last iteration cycle the IACTION block should be placed
    IACTION
    {
        // Refresh functional coverage
        dut.refresh_coverage();
        // Make one cycle of simulation and return contol to this point
        dut.cycle();

        // Receive results of the reference model work
        YIELD(dut.verdict());
    }
    //     }
    // }
    IEND
}

bool FIFOScenario::scen_push(Ctx& ctx)
{
    IBEGIN
    IACTION
    {
        // Precondition of the operation: iface1 is free and the FIFO reference model is not full
        if(dut.isIface1Ready() && !dut.is_full()) {
            // If the precondition is met one should create message to be sent and start corresponding stimulus
            InputData msg = InputData();
            dut.start(&FIFO::push_msg, dut.iface1, msg, FIFO::START_REQUEST_IFACE);
        }
        YIELD(dut.verdict());
    }
    IEND
}

bool FIFOScenario::scen_pop(cpptesk::ts::IntAbcCtx& ctx)
{
    IBEGIN
    IACTION
    {
        // Precondition of the operation: iface2 is free and the FIFO reference model is not empty
        if(dut.isIface2Ready() && !dut.is_empty()) {
            // If the precondition is met one should create message to be sent and start corresponding stimulus
            InputData msg = InputData();
            dut.start(&FIFO::pop_msg, dut.iface2, msg, FIFO::START_REQUEST_IFACE);
        }
        YIELD(dut.verdict());
    }
    IEND
}

std::string FIFOScenario::get_state_fifo()
{
    std::string state;
    std::stringstream out;

    out << "{fifo:" << dut.fifo_fullness() << ", write_started:" << dut.isIface1Ready() << ", read_started:" << dut.isIface2Ready() << "}";
    state = out.str();
#ifdef DEBUG
    std::cout << "get_state_fifo(): " << state << std::endl;
#endif
    return state;
}

FIFOScenario::FIFOScenario(void) : ScenarioBase()
{
}
FIFOScenario::~FIFOScenario(void) {}

ExampleScenario::ExampleScenario(void) : FIFOScenario()
{
    // Scenario initialization function. It requires exactly three parameters: init_function, finish_function, and get_current_state_function
    setup("FIFOScenario", &FIFOScenario::init, &FIFOScenario::finish, &FIFOScenario::get_state_fifo);

    // Scenario function registration. NOP should be the first in the list
    ADD_SCENARIO_METHOD(FIFOScenario::scen_nop);
    ADD_SCENARIO_METHOD(FIFOScenario::scen_push);
    ADD_SCENARIO_METHOD(FIFOScenario::scen_pop);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

